use std::fmt::Display;

use crate::OkId;

#[derive(Copy, Clone, Debug, PartialEq)]
pub(super) struct Uuid(u128);

impl From<uuid::Uuid> for OkId {
    fn from(value: uuid::Uuid) -> Self {
        Self {
            hash_type: super::BinaryType::Uuid,
            digest: super::Digest {
                uuid: Uuid(value.as_u128()),
            },
        }
    }
}

impl Display for Uuid {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let buf = self.0.to_be_bytes();
        // Write the hex digest
        for byte in buf {
            write!(f, "{:02x}", byte)?;
        }
        Ok(())
    }
}
